# EDBOOT - Dual Image Firmware Boot Loader
# Copyright (C) 2014 Softing Industrial Automation GmbH
# PATENT NO. DE 102010016257 A1

	.cpu cortex-a9
	.text
	.code 32

. = 0x0000
.globl _start
_start:
	b entry /* 0x00 reset */
	b .     /* 0x04 undefined instruction */
	b .     /* 0x08 software interrupt */
	b .     /* 0x0C abort prefetch */
	b .     /* 0x10 abort data */
	b .     /* 0x14 unused */
	b .     /* 0x18 irq */
	b .     /* 0x1C fiq */
except_end:

. = 0x0040
length = (_end-_start)/4
altera_header:
	.long  0x31305341 /* Validation word   */
	.byte  0x00       /* preloader version */
	.byte  0x00       /* flags - unused    */
	.short length     /* Program length    */
	.short 0x0000
	.short 0x31+0x30+0x53+0x41+(length & 0xFF)+(length>>8) /* checksum */

entry:	/* 0x4C Altera Boot ROM jumps to here */
	b reset

. = 0x0080
version:
	.ascii "EDBOOT V1.00.0.01.release\0"

. = 0x00C0
copyright:
	.ascii "Copyright (C) 2014-2015 Softing Industrial Automation GmbH\0"

# Manufacturer/device codes:
# 0001 = Softing
#     01 = RTEM
#     02 = FG200
#       00 = A4 and A6 prototype hardware
#       01 = A2 hardware
#       02 = Linux variant
#     03 = RTEM_V2
#     04 = CRTEM_V1
#     05 = CRTEM_V2
#     06 = FG260 (EIP-PN)
#       00 = A6 hardware
# 0002 = E+H
#     01 = Promass
# 0003 = BHI
#     01 = DAQ
# 0004 = Softing ProtocolIP evaluation boards
#     01 = Altera INK
#     02 = Altera CVE
#     03 = Altera CVSOC
#     04 = EBV DBC5
#
# NOTE: Please contact ED if you want to assign new manufacturer/device codes.

MAGIC_NUMBER  = 987654321
LOAD_ADDRESS  = 0x01000040
OFFSET_IMAGE1 = 0x00000000
MAX_IMAGESIZE = 0x000C0000

. = 0x0100
header:
	.long 987654321  /* magic number softing boot block           */
	.long 0x00010202 /* manuf. code 1 device 2 revision 2         */
	.long 0x80000000 /* start address image_1 (at block boundary) */
	.long 0x800D0000 /* start device tree (at block boundary)     */
	.long 0x800E0000 /* start linux firmware (at block boundary)  */
	.long 0
bootrom_data:
	.long 0
	.long 0

reset:
# save boot-rom data
	ldr r1,[r0,#0x38] /* saved rstmgr.stat */
	ldr r2,=bootrom_data
	str r1,[r2]

# freeze all IO banks
	mov r0,#0x00
	ldr r2,=0xFFD08000
	str r0,[r2,#0x40]
	str r0,[r2,#0x44]
	str r0,[r2,#0x48]
	mov r1,#0xE0
	str r1,[r2,#0x50]
	str r0,[r2,#0x54]

# reset all peripherals
	ldr r3,=0xFFD05000
	mov r1,#0x02
	str r1,[r3,#0x10] /* reset CPU1 */
	ldr r1,=0x3FFFFFFF
	str r1,[r3,#0x14]
	mov r1,#0xFF
	str r1,[r3,#0x18]
	mov r1,#0x07
	str r1,[r3,#0x1C]

# initialize clkmgr
# Main PLL VCO         1600MHz
# mpu_base_clk          800MHz (MAIN.C0)
# mpu_clk               800MHz
# mpu_l2_ram_clk        400MHz
# mpu_periph_clk        200MHz
# main_base_clk         400MHz (MAIN.C1)
# l3_main_clk           400MHz
# l4_main_clk           400MHz
# l3_mp_clk             200MHz
# l3_sp_clk             100MHz
# dbg_base_clk          400MHz (MAIN.C2)
# dbg_at_clk            400MHz
# dbg_clk               200MHz
# dbg_trace_clk         400MHz
# dbg_timer_clk         400MHz
# main_qspi_clk         400MHz (MAIN.C3)
# qspi_clk              400MHz
# main_nand_sdmmc_clk   200MHz (MAIN.C4)
# nand_x_clk            200MHz
# nand_clk               50MHz
# cfg_s2f_user0_clk     100MHz (MAIN.C5)
# cfg_clk               100MHz
# Peripheral PLL VCO   1000MHz
# emac0_clk             250MHz (PER.C0)
# emac1_clk             250MHz (PER.C1)
# periph_qspi_clk       500MHz (PER.C2)
# periph_nand_sdmmc_clk 200MHz (PER.C3)
# sdmmc_clk             200MHz
# periph_base_clk       200MHz (PER.C4)
# l4_mp_clk             100MHz
# l4_sp_clk             100MHz
# usb_mp_clk            200MHz
# spi_m_clk             200MHz
# can0_clk              100MHz
# can1_clk              100MHz
# gpio_clk               32KHz
# s2f_user1_clk         100MHz (PER.C5)
# SDRAM PLL VCO         600MHz
# ddr_dqs_clk           300MHz (SDR.C0)
# ddr_2x_dqs_clk        600MHz (SDR.C1)
# ddr_dq_clk            300MHz (SDR.C2)
# s2f_user2_clk         100MHz (SDR.C5)

# switch all clocks OFF
	ldr r4,=0xFFD04000
	mov r1,#0x200
	str r1,[r4,#0xA0]
	mov r1,#0x30
	str r1,[r4,#0x60]
	str r0,[r4,#0xD8]
	bl wait0
	str r0,[r4,#0xA0]
# clkmgr.bypass: ALL ON
	mov r1,#0x0B
	str r1,[r4,#0x04]
	bl wbusy
# double-reset clkmgr.sdrpllgrp.vco
	ldr r1,=0x8001000D
	str r1,[r4,#0xC0]
	mov r1,#0x5
	bl wait0
	ldr r1,=0x800000C0
	str r1,[r4,#0xC0]
	mov r1,#0x7
	bl wait0
	ldr r1,=0x000000C2
	str r1,[r4,#0xC0]
# stop all PLLs
	ldr r1,=0x8001000D
	str r1,[r4,#0x40]
	str r1,[r4,#0x80]
	str r1,[r4,#0xC0]
	mov r1,#0x15
	str r1,[r4,#0xAC]
	str r0,[r4,#0x70]
	ldr r1,[r4,#0x40]
	ldr r1,[r4,#0x80]
	ldr r1,[r4,#0xC0]
# start all PLLs
	ldr r1,=0x80020638
	str r1,[r4,#0x40]
	ldr r1,=0x800203E0
	str r1,[r4,#0x80]
	ldr r1,=0x800000C0
	str r1,[r4,#0xC0]
# clkmgr.mainpllgrp.mpuclk
	str r0,[r4,#0x48]
# clkmgr.mainpllgrp.mainclk
	str r0,[r4,#0x4C]
# clkmgr.mainpllgrp.dbgatclk
	str r0,[r4,#0x50]
# clkmgr.mainpllgrp.mainqspiclk
	mov r1,#0x03
	str r1,[r4,#0x54]
# clkmgr.mainpllgrp.mainnandsdmmcclk
	mov r1,#0x07
	str r1,[r4,#0x58]
# clkmgr.mainpllgrp.cfgs2fuser0clk
	mov r1,#0x0F
	str r1,[r4,#0x5C]
# clkmgr.perpllgrp.emac0clk
	mov r1,#0x03
	str r1,[r4,#0x88]
# clkmgr.perpllgrp.emac1clk
	str r1,[r4,#0x8C]
# clkmgr.perpllgrp.perqspiclk
	mov r1,#0x01
	str r1,[r4,#0x90]
# clkmgr.perpllgrp.pernandsdmmcclk
	mov r1,#0x04
	str r1,[r4,#0x94]
# clkmgr.perpllgrp.perbaseclk
	str r1,[r4,#0x98]
# clkmgr.perpllgrp.s2fuser1clk
	mov r1,#0x09
	str r1,[r4,#0x9C]
# clkmgr.sdrpllgrp.ddrdqsclk
	mov r1,#0x01
	str r1,[r4,#0xC8]
# clkmgr.sdrpllgrp.ddr2xdqsclk
	str r0,[r4,#0xCC]
# clkmgr.sdrpllgrp.ddrdqclk
	str r1,[r4,#0xD0]
# clkmgr.sdrpllgrp.s2fuser2clk
	mov r1,#0x05
	str r1,[r4,#0xD4]
# enable VCO
	ldr r1,=0x0002063A
	str r1,[r4,#0x40]
	ldr r1,=0x000203E2
	str r1,[r4,#0x80]
	ldr r1,=0x000000C2
	str r1,[r4,#0xC0]
# clkmgr.mainpllgrp.maindiv
	mov r1,#0x95
	str r1,[r4,#0x64]
# clkmgr.mainpllgrp.dbgdiv
	mov r1,#0x04
	str r1,[r4,#0x68]
# clkmgr.mainpllgrp.tracediv
	str r0,[r4,#0x6C]
# clkmgr.mainpllgrp.l4src
	mov r1,#0x03
	str r1,[r4,#0x70]
# clkmgr.perpllgrp.div
	ldr r1,=0x00000240
	str r1,[r4,#0xA4]
# clkmgr.perpllgrp.gpiodiv
	ldr r1,=0x00001869
	str r1,[r4,#0xA8]
# clkmgr.perpllgrp.src
	mov r1,#0x16
	str r1,[r4,#0xAC]
# wait until PLLs are stable
	bl wait1
# set outresetall
	ldr r1,=0x0102063A
	str r1,[r4,#0x40]
	ldr r1,=0x010203E2
	str r1,[r4,#0x80]
	ldr r1,=0x010000C2
	str r1,[r4,#0xC0]
# clear outresetall
	ldr r1,=0x0002063A
	str r1,[r4,#0x40]
	ldr r1,=0x000203E2
	str r1,[r4,#0x80]
	ldr r1,=0x000000C2
	str r1,[r4,#0xC0]
# apply 1/4 clock phase offset to ddrdqclk
	ldr r1,=0x00000801
	str r1,[r4,#0xD0]
phase1:
	ldr r1,[r4,#0xD0]
	cmp r1,#0x01
	bne phase1
# clkmgr.bypass: ALL OFF
	str r0,[r4,#0x04]
	bl wbusy
# clkmgr.ctrl: Safe Mode OFF
	mov r1,#0x05
	str r1,[r4]
	bl wbusy
# clkmgr.dbctrl: Safe Mode OFF
	str r0,[r4,#0x10]
	bl wbusy
# enable all clocks
	mov r1,#0x3F
	str r1,[r4,#0x60]
	ldr r1,=0x00000A0C
	str r1,[r4,#0xA0]
	mov r1,#0x07
	str r1,[r4,#0xD8]

# initialize HPS VIO bank 0+1
	mov r1,#0x01
	ldr r4,=0xFFF02000
	str r1,[r4,#0x04]
	ldr r5,=csr0_data
	ldr r6,=csr0_len
	bl loadcsr

# initialize HPS VIO bank 2+3 (QSPI)
	mov r1,#0x02
	str r1,[r4,#0x04]
	ldr r5,=csr1_data
	ldr r6,=csr1_len
	bl loadcsr

# initialize HPS VIO bank 4
	mov r1,#0x04
	str r1,[r4,#0x04]
	ldr r5,=csr2_data
	ldr r6,=csr2_len
	bl loadcsr

# initialize HPS HIO (DDR3)
	mov r1,#0xC0
	str r1,[r2,#0x50]
	mov r1,#0x08
	str r1,[r4,#0x04]
	ldr r5,=csr3_data
	ldr r6,=csr3_len
	bl loadcsr

# initialize pinmux for QSPI
	str r0,[r2,#0x500]
	str r0,[r2,#0x534]
	mov r1,#0x02
	str r1,[r2,#0x538]
	mov r1,#0x03
	str r1,[r2,#0x53C]
	str r1,[r2,#0x540]
	str r1,[r2,#0x544]
	str r1,[r2,#0x548]
	str r1,[r2,#0x54C]
	str r1,[r2,#0x550]

# reset off for DDR3 and QSPI
	ldr r1,=0x1FFFFFDF
	str r1,[r3,#0x14]

# unfreeze I/O
	mov r1,#0x03
	str r1,[r2,#0x40]
	str r1,[r2,#0x44]
	str r1,[r2,#0x48]
	mov r1,#0x0F
	str r1,[r2,#0x40]
	str r1,[r2,#0x44]
	str r1,[r2,#0x48]
	mov r1,#0x1F
	str r1,[r2,#0x40]
	str r1,[r2,#0x44]
	str r1,[r2,#0x48]
	mov r1,#0x1C0
	str r1,[r2,#0x50]
	bl wait1
	ldr r1,=0x183
	str r1,[r2,#0x50]
	bl wait1
	ldr r1,=0x18F
	str r1,[r2,#0x50]
	ldr r1,=0x10F
	str r1,[r2,#0x50]
	ldr r1,=0x11F
	str r1,[r2,#0x50]

# initialize DDR3
	ldr r5,=ddr3_data
	ldr r6,=ddr3_len
	bl loadseq

# sweep all ECC enabled ram
	ldr r1,=_end
sweep:
	str r0,[r1],#4
	cmp r1,#0
	bne sweep

# fully enable ECC
	dmb
	mov r1,#0x1
	str r1,[r2,#0x140] /* sysmgr.eccgroup.L2 */
# clear ECC error bits
	ldr r1,[r2,#0x144] /* sysmgr.eccgroup.ocram */
	str r1,[r2,#0x144] /* reset OCRAM ECC error */

# preloader has reached a valid state
	ldr r1,=0x49535756 /* magic value: 'ISWV' */
	str r1,[r2,#0xC8]  /* sysmgr.romcodegrp.initswstate */

# init exception vectors
	dmb
	ldr r1,[pc,#_start+4-.-8]
	mov r2,#0
loop:
	str r1,[r2],#4
	cmp r2,#except_end-_start
	bne loop

# initialize QSPI
	ldr r1,=0x00083486
	ldr r4,=0xFF705000
	str r1,[r4]
	ldr r1,=0x0A0220EB /* fast read quad/quad */
	str r1,[r4,#0x04]
	ldr r1,=0x00022012 /* page program quad/quad */
	str r1,[r4,#0x08]
	mov r1,#0x10000000
	str r1,[r4,#0x0C]
	mov r1,#0x09
	str r1,[r4,#0x10]
	ldr r1,=0x00101002
	str r1,[r4,#0x14]
	ldr r1,=0x00083487
	str r1,[r4]
	dmb

init:
	mov r1,#0x66000000 /* reset enable */
	bl wqspi
	mov r1,#0x99000000 /* reset memory */
	bl wqspi
	ldr r1,=0x70800000 /* read flag status word */
	bl wqspi
	ldr r1,[r4,#0xA0]
	tst r1,#0x80 /* busy bit */
	beq init
	tst r1,#0x01 /* 4 byte bit */
	bne init

# boot firmware
boot:
	ldr r2,=MAX_IMAGESIZE/4
	ldr r5,=0xFFA00000
	ldr r6,=LOAD_ADDRESS
copy:
	ldr r1,[r5],#4
	str r1,[r6],#4
	sub r2,r2,#1
	cmp r2,#0
	bne copy

# disable QSPI again
	ldr r2,=0x00083486
	str r2,[r4]

jump:
	ldr pc,=LOAD_ADDRESS
# done

wqspi:
	str r1,[r4,#0x90]
	orr r1,r1,#0x01
	str r1,[r4,#0x90]
wqspi1:
	ldr r1,[r4,#0x90]
	tst r1,#0x02
	bne wqspi1
	mov pc,lr

wait1:
	mov r1,#0x10000
wait0:
	sub r1,r1,#1
	cmp r1,#0
	bne wait0
	mov pc,lr

wbusy:
	ldr r1,[r4,#0x14]
	tst r1,#0x01
	bne wbusy
	mov pc,lr

loadcsr:
	ldr r1,[r5],#0x04
	cmp r6,#0x04
	ble lastwd
	str r1,[r4,#0x1C]
	sub r6,r6,#0x04
	b loadcsr
lastwd:
	add r6,r6,#0x03
	str r1,[r4,r6,lsl #2]
wfifo:
	ldr r1,[r4]
	tst r1,#0xF0000000
	bne wfifo
	str r0,[r4,#0x04]
	mov pc,lr

loadseq:
	ldr r1,[r5],#0x04
	ldr r7,[r5],#0x04
	str r1,[r7]
	sub r6,r6,#0x01
	cmp r6,#0x00
	bne loadseq
	mov pc,lr

	.pool

	.include "csr0_data.i"
	.include "csr1_data.i"
	.include "csr2_data.i"
	.include "csr3_data.i"

ddr3_len=(ddr3_data_end-ddr3_data)/8
ddr3_data:
	.long 0x000A8042, 0xFFC25000
	.long 0x4E310A05, 0xFFC25004
	.long 0x08AAA492, 0xFFC25008
	.long 0x00221EB4, 0xFFC2500C
	.long 0x00402824, 0xFFC25010
	.long 0x00000000, 0xFFC25014
	.long 0x00000001, 0xFFC25018
	.long 0x00000DEA, 0xFFC2502C
	.long 0x00000020, 0xFFC25030
	.long 0x00000008, 0xFFC25034
	.long 0x00000000, 0xFFC2503C
	.long 0x00000002, 0xFFC2505C
	.long 0x00000002, 0xFFC25060
	.long 0x00044555, 0xFFC25064
	.long 0x2C011000, 0xFFC25068
	.long 0x00B00088, 0xFFC2506C
	.long 0x00760210, 0xFFC25070
	.long 0x00980543, 0xFFC25074
	.long 0x0005A56A, 0xFFC25078
	.long 0x0000003F, 0xFFC2507C
	.long 0x00000000, 0xFFC25088
	.long 0x3FFD1088, 0xFFC250AC
	.long 0x21084210, 0xFFC250B0
	.long 0x8081EF84, 0xFFC250B4
	.long 0x00000000, 0xFFC250B8
	.long 0x0000F800, 0xFFC250BC
	.long 0x20820820, 0xFFC250C0
	.long 0x08208208, 0xFFC250C4
	.long 0x41041041, 0xFFC250C8
	.long 0x00410410, 0xFFC250CC
	.long 0x01010101, 0xFFC250D0
	.long 0x01010101, 0xFFC250D4
	.long 0x00000101, 0xFFC250D8
	.long 0x0000000A, 0xFFC2505C
	.include "sequencer_data.i"
	.long 0x00000001, 0xFFFEFC00
	.long 0xC0000000, 0xFFFEFC04
ddr3_data_end:

	.long 0xFFFFFFFF /* placeholder for CRC */
_end:	/* End Of Image */
