#include <common.h>
#include <u-boot/zlib.h>
#include <asm/arch/fpga_manager.h>

extern const unsigned char *fpga_firmware_gz;
extern unsigned long        fpga_firmware_len;

//=============================================================================

int hal_load_fpga_firmware(void)
{
  z_stream stream;
  unsigned long uc_len = 0x700000;
  /* Note: malloc(uc_len) would fail and return NULL.
     Overwrite the original load address instead. */
  void *uc_data = (void*) CONFIG_SYS_TEXT_BASE;
  int err;

  stream.next_in = (Bytef*)fpga_firmware_gz;
  stream.avail_in = fpga_firmware_len;

  stream.next_out = uc_data;
  stream.avail_out = uc_len;

  stream.zalloc = Z_NULL;
  stream.zfree = Z_NULL;
  stream.opaque = Z_NULL;

  err = inflateInit2(&stream, 31);
  if (err != Z_OK)
  {
    return 1;
  }

  err = inflate(&stream, Z_PARTIAL_FLUSH);
  if (err != Z_STREAM_END)
  {
    inflateEnd(&stream);
    return 1;
  }

  err = inflateEnd(&stream);
  if (err != Z_OK)
  {
    return 1;
  }

  err = fpgamgr_program_fpga(uc_data, stream.total_out);
  return err;
}
