/*
 *  Copyright Altera Corporation (C) 2013. All rights reserved
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms and conditions of the GNU General Public License,
 *  version 2, as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 *  more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/system_manager.h>
#include <asm/arch/reset_manager.h>
#ifndef CONFIG_SPL_BUILD
#include <asm/arch-socfpga/nic301.h>
#include <phy.h>
#include <miiphy.h>
#include <netdev.h>
#include "../../../drivers/net/designware.h"
#endif
#include <i2c.h>
#include <libfdt.h>

DECLARE_GLOBAL_DATA_PTR;

/*
 * Initialization function which happen at early stage of c code
 */
int board_early_init_f(void)
{
#ifdef CONFIG_HW_WATCHDOG
	/* disable the watchdog when entering U-Boot */
	watchdog_disable();
#endif
	/* calculate the clock frequencies required for drivers */
	cm_derive_clocks_for_drivers();

#ifndef CONFIG_SPL_BUILD
	/* initialize PINMUX functions */
	writel(2,SYSMGR_PINMUXGRP_GENERALIO(15)); /* UART1_RX */
	writel(2,SYSMGR_PINMUXGRP_GENERALIO(16)); /* UART1_TX */

	/* enable clock for FPGA MANAGER */
	setbits_le32(SOCFPGA_CLKMGR_ADDRESS + CLKMGR_MAINPLLGRP_EN_ADDRESS,
		     0x100);
	/* enable clock for GPIO,SDMMC,EMAC0-1 */
	setbits_le32(SOCFPGA_CLKMGR_ADDRESS + CLKMGR_PERPLLGRP_EN_ADDRESS,
		     0x143);
	/* reset disable for DMA,GPIO0-2,UART1,I2C0,SPTIMER0-1,OSC1TIMER0-1 */
	clrbits_le32(SOCFPGA_RSTMGR_ADDRESS + RSTMGR_PERMODRST_OFFSET,
		     0x1E021F00);
#endif
	return 0;
}

/*
 * Miscellaneous platform dependent initialisations
 */
int board_init(void)
{
	/* adress of boot parameters for ATAG (if ATAG is used) */
	gd->bd->bi_boot_params = 0x00000100;

	/*
	 * reinitialize the global variable for clock value as after
	 * relocation, the global variable are cleared to zeroes
	 */
	cm_derive_clocks_for_drivers();
	return 0;
}

static void setenv_ethaddr_eeprom(void)
{
#if 0
	uint addr, alen;
	int linebytes;
	uchar chip, enetaddr[6];

	/* configuration based on FG200 EEPROM */
	chip = 0x50;            /* slave ID for EEPROM */
	alen = 1;               /* AT24C01 using 1 byte addressing */
	linebytes = 6;          /* emac address stored in 6 bytes address */

#if (CONFIG_EMAC_BASE == CONFIG_EMAC0_BASE)
	addr = 0x20;
#elif (CONFIG_EMAC_BASE == CONFIG_EMAC1_BASE)
	addr = 0x28;
#endif

	if (i2c_read(chip, addr, alen, enetaddr, linebytes) == 0 &&
	    is_valid_ether_addr(enetaddr))
		eth_setenv_enetaddr("ethaddr", enetaddr);
	else
#endif
		puts("Skipped ethaddr assignment due to invalid "
			"EMAC address in EEPROM\n");
}

static void i2c_soft_reset(void)
{
#define I2C_SDA_PIN 55  /* at GPIO_55 */
#define I2C_SCL_PIN 56  /* at GPIO_56 */

	/* switch SDA+SCL to GPIO, avoid glitches */
	SOCFPGA_GPIO_INPUT(I2C_SDA_PIN);
	SOCFPGA_GPIO_INPUT(I2C_SCL_PIN);
	writel(0,SYSMGR_PINMUXGRP_GPLINMUX(I2C_SDA_PIN));
	writel(0,SYSMGR_PINMUXGRP_GPLINMUX(I2C_SCL_PIN));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(I2C_SDA_PIN));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(I2C_SCL_PIN));
	writel(0,SYSMGR_PINMUXGRP_GENERALIO(7)); /* GPIO_55 */
	writel(0,SYSMGR_PINMUXGRP_GENERALIO(8)); /* GPIO_56 */

	while (SOCFPGA_GPIO_READ(I2C_SCL_PIN) == 0);
	udelay(5);
	while (SOCFPGA_GPIO_READ(I2C_SDA_PIN) == 0) {
		puts("I2C: SDA locked\n");
		SOCFPGA_GPIO_LOW(I2C_SCL_PIN);
		udelay(5);
		SOCFPGA_GPIO_INPUT(I2C_SCL_PIN);
		while (SOCFPGA_GPIO_READ(I2C_SCL_PIN) == 0);
		udelay(5);
	}

	/* now switch SDA+SCL to I2C0 */
	writel(1,SYSMGR_PINMUXGRP_GENERALIO(7)); /* I2C0_SDA */
	writel(1,SYSMGR_PINMUXGRP_GENERALIO(8)); /* I2C0_SCL */
}

#define USD_CLK1_PIN 45
#define USD_CLK2_PIN 37

static void sdmmc_soft_reset(int clk_pin)
{
#define USD_CMD_PIN 36

	/* first 74 clocks preamble, then
	   CMD12: STOP_TRANSMISSION plus 74 clocks for response, then
	   CMD7 (RCA=0): DESELECT_CARD plus 8 clocks for switching.  */
	char *cmdbits = "11111111111111111111111111111111111111111111111111111111111111111111111111"
			"1010011000000000000000000000000000000000001100001"
			"XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"
			"1010001110000000000000000000000000000000010000011"
			"XXXXXXXX";

	/* USD_CMD <= SDMMC_CMD */
	SOCFPGA_GPIO_INPUT(USD_CMD_PIN);
	writel(0,SYSMGR_PINMUXGRP_FLASHIO(0));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(USD_CMD_PIN));

	/* USD_DAT0 <= SDMMC_DATA(0) */
	SOCFPGA_GPIO_INPUT(38);
	writel(0,SYSMGR_PINMUXGRP_FLASHIO(2));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(38));

	/* USD_DAT1 <= SDMMC_DATA(1) */
	SOCFPGA_GPIO_INPUT(39);
	writel(0,SYSMGR_PINMUXGRP_FLASHIO(3));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(39));

	/* USD_DAT2 <= SDMMC_DATA(2) */
	SOCFPGA_GPIO_INPUT(46);
	writel(0,SYSMGR_PINMUXGRP_FLASHIO(10));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(46));

	/* USD_CD <= SDMMC_DATA(3) */
	SOCFPGA_GPIO_INPUT(47);
	writel(0,SYSMGR_PINMUXGRP_FLASHIO(11));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(47));

	/* USD_CLK1 <= SDMMC_CCLK_OUT(1) */
	SOCFPGA_GPIO_LOW(USD_CLK1_PIN);
	writel(0,SYSMGR_PINMUXGRP_FLASHIO(9));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(USD_CLK1_PIN));

	/* USD_CLK2 <= SDMMC_CCLK_OUT(2) */
	SOCFPGA_GPIO_LOW(USD_CLK2_PIN);
	writel(0,SYSMGR_PINMUXGRP_FLASHIO(1));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(USD_CLK2_PIN));

	while (*cmdbits)
	{
		switch (*cmdbits++)
		{
			case '0':
				SOCFPGA_GPIO_LOW(USD_CMD_PIN);
				break;
			case '1':
				SOCFPGA_GPIO_HIGH(USD_CMD_PIN);
				break;
			default:
				SOCFPGA_GPIO_INPUT(USD_CMD_PIN);
				break;
		}
		SOCFPGA_GPIO_HIGH(clk_pin);
		udelay(2);
		SOCFPGA_GPIO_LOW(clk_pin);
		udelay(2);
	}
}

static void disable_hardware_watchdog(void)
{
#define WDI_PIN 3  /* at GPIO_3 */

	/* disable HW-Watchdog */
	SOCFPGA_GPIO_INPUT(WDI_PIN);
	writel(0,SYSMGR_PINMUXGRP_EMACIO(WDI_PIN)); /* GPIO_3 */
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(WDI_PIN));
}

static void initialize_gpio_pins(void)
{
	/* LED_RUN_R <= 0 */
	SOCFPGA_GPIO_LOW(0);
	writel(0,SYSMGR_PINMUXGRP_EMACIO(0));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(0));

	/* LED_RUN_G <= 0 */
	SOCFPGA_GPIO_LOW(2);
	writel(0,SYSMGR_PINMUXGRP_EMACIO(2));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(2));

	/* TRIGGER_OUT <= 1 */
	SOCFPGA_GPIO_HIGH(5);
	writel(0,SYSMGR_PINMUXGRP_EMACIO(5));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(5));

	/* LED_ERR_G <= 0 */
	SOCFPGA_GPIO_LOW(8);
	writel(0,SYSMGR_PINMUXGRP_EMACIO(8));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(8));

	/* LED_ERR_R <= 0 */
	SOCFPGA_GPIO_LOW(10);
	writel(0,SYSMGR_PINMUXGRP_EMACIO(10));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(10));

	/* LED_SYS_R <= 0 */
	SOCFPGA_GPIO_LOW(12);
	writel(0,SYSMGR_PINMUXGRP_EMACIO(12));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(12));

	/* USD2_CDETECT <= 'Z' */
	SOCFPGA_GPIO_INPUT(18);
	writel(0,SYSMGR_PINMUXGRP_MIXED1IO(4));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(18));

	/* LED_SYS_G <= 0 */
	SOCFPGA_GPIO_LOW(22);
	writel(0,SYSMGR_PINMUXGRP_MIXED1IO(8));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(22));

	/* FACTORY_RESET <= 'Z' */
	SOCFPGA_GPIO_INPUT(27);
	writel(0,SYSMGR_PINMUXGRP_MIXED1IO(13));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(27));

	/* USD_CMD <= SDMMC_CMD */
	writel(0,SYSMGR_PINMUXGRP_FLASHIO(0));
	writel(0,SYSMGR_PINMUXGRP_GPLMUX(36));

	/* USD_CLK2 <= SDMMC_CCLK_OUT(2) */
	writel(0,SYSMGR_PINMUXGRP_FLASHIO(1));
	writel(0,SYSMGR_PINMUXGRP_GPLMUX(37));

	/* USD_DAT0 <= SDMMC_DATA(0) */
	writel(0,SYSMGR_PINMUXGRP_FLASHIO(2));
	writel(0,SYSMGR_PINMUXGRP_GPLMUX(38));

	/* USD_DAT1 <= SDMMC_DATA(1) */
	writel(0,SYSMGR_PINMUXGRP_FLASHIO(3));
	writel(0,SYSMGR_PINMUXGRP_GPLMUX(39));

	/* USD_CLK1 <= SDMMC_CCLK_OUT(1) */
	writel(0,SYSMGR_PINMUXGRP_FLASHIO(9));
	writel(0,SYSMGR_PINMUXGRP_GPLMUX(45));

	/* USD_DAT2 <= SDMMC_DATA(2) */
	writel(0,SYSMGR_PINMUXGRP_FLASHIO(10));
	writel(0,SYSMGR_PINMUXGRP_GPLMUX(46));

	/* USD_CD <= SDMMC_DATA(3) */
	writel(0,SYSMGR_PINMUXGRP_FLASHIO(11));
	writel(0,SYSMGR_PINMUXGRP_GPLMUX(47));

	/* USD1_CDETECT <= 'Z' */
	SOCFPGA_GPIO_INPUT(57);
	writel(0,SYSMGR_PINMUXGRP_GENERALIO(9));
	writel(0,SYSMGR_PINMUXGRP_GPLINMUX(57));
	writel(1,SYSMGR_PINMUXGRP_GPLMUX(57));
}

#ifdef CONFIG_BOARD_LATE_INIT
int board_late_init(void)
{
	uchar enetaddr[6];

	disable_hardware_watchdog();
	i2c_soft_reset();
	sdmmc_soft_reset(USD_CLK1_PIN);
	sdmmc_soft_reset(USD_CLK2_PIN);
	initialize_gpio_pins();

	/* if no ethaddr environment, get it from EEPROM */
	if (!eth_getenv_enetaddr("ethaddr", enetaddr))
		setenv_ethaddr_eeprom();
	return 0;
}
#endif

int hal_load_fpga_firmware(void);

/* We know all the init functions have been run now */
int board_eth_init(bd_t *bis)
{
#if !defined(CONFIG_SOCFPGA_VIRTUAL_TARGET) && \
!defined(CONFIG_SPL_BUILD)

	/* Initialize EMAC */

	/*
	 * Putting the EMAC controller to reset when configuring the PHY
	 * interface select at System Manager
	*/
	emac0_reset_enable(1);
	emac1_reset_enable(1);

	/* Load an fpga design to route the EMAC */
	if(hal_load_fpga_firmware())
	{
		puts("FPGA firmware error\n");
		return 0;
	}

	/* Pulse HPS to FPGA warm reset signal */
	writel(0x40, SOCFPGA_RSTMGR_ADDRESS + RSTMGR_MISCMODRST_OFFSET);
	udelay(100);
	writel(0, SOCFPGA_RSTMGR_ADDRESS + RSTMGR_MISCMODRST_OFFSET);

	do {
		udelay(100);
	} while((readl(SOCFPGA_FPGAMGRREGS_ADDRESS + 0x14) & 3) == 0);

	if((readl(SOCFPGA_FPGAMGRREGS_ADDRESS + 0x14) & 3) == 2)
	{
		/* allow 100ms to recover from a glitch */
		udelay(100000);
	}

	if((readl(SOCFPGA_FPGAMGRREGS_ADDRESS + 0x14) & 1) == 0)
	{
		puts("Security EEPROM error\n");
		return 0;
	}

	/* route EMAC to FPGA */
	writel(1,SYSMGR_PINMUXGRP_EMAC0USEFPGA);
	writel(1,SYSMGR_PINMUXGRP_EMAC1USEFPGA);
	writel(1,SYSMGR_PINMUXGRP_SDMMCUSEFPGA);

	/* route EMAC0 to ETH1/PHY pins */
	setbits_le32(SOCFPGA_FPGAMGRREGS_ADDRESS + 0x10, 0x1);

	/* Setting sysmgr.fpgaintfgrp.module.emac[0:1] = "11" */
	/* undocumented bit 0x20 enables SDMMC CARD_DETECT signal */
	setbits_le32(SYSMGR_FPGAINTF_MODULE, 0x2C);

	/* configure to PHY interface select choosed */
	writel(
#if (CONFIG_PHY_INTERFACE_MODE == SOCFPGA_PHYSEL_ENUM_GMII)
		(SYSMGR_EMACGRP_CTRL_PHYSEL_ENUM_GMII_MII <<
		SYSMGR_EMACGRP_CTRL_PHYSEL0_LSB) |
		(SYSMGR_EMACGRP_CTRL_PHYSEL_ENUM_GMII_MII <<
		SYSMGR_EMACGRP_CTRL_PHYSEL1_LSB),
#elif (CONFIG_PHY_INTERFACE_MODE == SOCFPGA_PHYSEL_ENUM_MII)
		(SYSMGR_EMACGRP_CTRL_PHYSEL_ENUM_GMII_MII <<
		SYSMGR_EMACGRP_CTRL_PHYSEL0_LSB) |
		(SYSMGR_EMACGRP_CTRL_PHYSEL_ENUM_GMII_MII <<
		SYSMGR_EMACGRP_CTRL_PHYSEL1_LSB),
#elif (CONFIG_PHY_INTERFACE_MODE == SOCFPGA_PHYSEL_ENUM_RGMII)
		(SYSMGR_EMACGRP_CTRL_PHYSEL_ENUM_RGMII <<
		SYSMGR_EMACGRP_CTRL_PHYSEL0_LSB) |
		(SYSMGR_EMACGRP_CTRL_PHYSEL_ENUM_RGMII <<
		SYSMGR_EMACGRP_CTRL_PHYSEL1_LSB),
#elif (CONFIG_PHY_INTERFACE_MODE == SOCFPGA_PHYSEL_ENUM_RMII)
		(SYSMGR_EMACGRP_CTRL_PHYSEL_ENUM_RMII <<
		SYSMGR_EMACGRP_CTRL_PHYSEL0_LSB) |
		(SYSMGR_EMACGRP_CTRL_PHYSEL_ENUM_RMII <<
		SYSMGR_EMACGRP_CTRL_PHYSEL1_LSB),
#endif
		CONFIG_SYSMGR_EMAC_CTRL);

	/* SDMMC needs this for access to SDRAM */
	writel(0x1, SOCFPGA_L3REGS_ADDRESS + L3REGS_SECGRP_SDRDATA_ADDRESS);

	/* reset disable SDMMC */
	clrbits_le32(SOCFPGA_RSTMGR_ADDRESS + RSTMGR_PERMODRST_OFFSET,
		     RSTMGR_PERMODRST_SDMMC_SET(1));

	/* Release the EMAC controller from reset */
#if (CONFIG_EMAC_BASE == CONFIG_EMAC0_BASE)
	emac0_reset_enable(0);
#elif (CONFIG_EMAC_BASE == CONFIG_EMAC1_BASE)
	emac1_reset_enable(0);
#endif

	/* initialize and register the emac */
	int rval = designware_initialize(0, CONFIG_EMAC_BASE,
		CONFIG_EPHY_PHY_ADDR,
#if (CONFIG_PHY_INTERFACE_MODE == SOCFPGA_PHYSEL_ENUM_GMII)
		PHY_INTERFACE_MODE_GMII);
#elif (CONFIG_PHY_INTERFACE_MODE == SOCFPGA_PHYSEL_ENUM_MII)
		PHY_INTERFACE_MODE_MII);
#elif (CONFIG_PHY_INTERFACE_MODE == SOCFPGA_PHYSEL_ENUM_RGMII)
		PHY_INTERFACE_MODE_RGMII);
#elif (CONFIG_PHY_INTERFACE_MODE == SOCFPGA_PHYSEL_ENUM_RMII)
		PHY_INTERFACE_MODE_RMII);
#endif
	debug("board_eth_init %d\n", rval);
	return rval;
#else
	return 0;
#endif
}

#if defined(CONFIG_OF_LIBFDT) && defined(CONFIG_OF_BOARD_SETUP)
void ft_board_setup(void *fdt, bd_t *bd)
{
	int nodeoffset;

	/* Find the "chosen" node.  */
	nodeoffset = fdt_path_offset (fdt, "/chosen");

	/* If there is a "chosen" node, add a preloader-info property. */
	if (nodeoffset >= 0)
		fdt_setprop(fdt, nodeoffset, "preloader-info", (void*)0xFFFF0100, 0x20);
}
#endif
